@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2014 The University of Utah
+// Copyright (c) 2012, 2014, 2015, 2017, 2018 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -23,7 +23,6 @@
 #include "TransformationManager.h"
 
 using namespace clang;
-using namespace llvm;
 
 static const char *DescriptionMsg =
 "Reduce the size of an array to the maximum index of \
@@ -121,9 +120,9 @@ void ReduceArraySize::doAnalysis(void)
     unsigned int DimSz = DimVec->size();
     TransAssert((DimSz == OrigDimVec->size()) &&
                 "Two DimValueVectors should have the same size!");
-    for (unsigned int I = 0; I < DimSz; ++I) {
-      int DimV = (*DimVec)[I];
-      int OrigDimV = (*OrigDimVec)[I];
+    for (unsigned int II = 0; II < DimSz; ++II) {
+      int DimV = (*DimVec)[II];
+      int OrigDimV = (*OrigDimVec)[II];
       if ((DimV == -1) || (OrigDimV == 0) || ((DimV+1) == OrigDimV))
         continue;
 
@@ -133,7 +132,7 @@ void ReduceArraySize::doAnalysis(void)
 
       TheVarDecl = VD;
       TheDimValue = DimV;
-      TheDimIdx = I;
+      TheDimIdx = II;
     }
   }
 }
@@ -183,6 +182,9 @@ void ReduceArraySize::rewriteArrayVarDecl(void)
 
 void ReduceArraySize::handleOneVar(const VarDecl *VD)
 {
+  if (isInIncludedFile(VD))
+    return;
+
   const Type *Ty = VD->getType().getTypePtr();
   const ArrayType *ArrayTy = dyn_cast<ArrayType>(Ty);
   if (!ArrayTy)
