@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013, 2014, 2015, 2016, 2017 The University of Utah
+// Copyright (c) 2012, 2013, 2014, 2015, 2016, 2017, 2019 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -23,7 +23,6 @@
 #include "TransformationManager.h"
 
 using namespace clang;
-using namespace llvm;
 
 static const char *DescriptionMsg =
 "Another pass to increase readability of reduced code. \
@@ -253,7 +252,7 @@ bool RenameCXXMethodVisitor::VisitDeclRefExpr(DeclRefExpr *DRE)
       &QualLoc, MD, NewName);
   }
   else {
-    ConsumerInstance->TheRewriter.ReplaceText(DRE->getLocStart(),
+    ConsumerInstance->TheRewriter.ReplaceText(DRE->getBeginLoc(),
       MD->getNameAsString().size(), NewName);
   }
 
@@ -535,7 +534,8 @@ bool RenameCXXMethod::isSpecialCXXMethod(const CXXMethodDecl *MD)
       dyn_cast<CXXConversionDecl>(MD))
     return true;
 
-  if (MD->isUsualDeallocationFunction() ||
+  SmallVector<const FunctionDecl*, 4> PreventedBy;
+  if (MD->isUsualDeallocationFunction(PreventedBy) ||
       MD->isCopyAssignmentOperator() ||
       MD->isMoveAssignmentOperator() ||
       MD->isLambdaStaticInvoker() ||
