@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013, 2014, 2015, 2016, 2017 The University of Utah
+// Copyright (c) 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -212,10 +212,11 @@ const Expr *Transformation::getArrayBaseExprAndIdxs(
   while (ASE) {
     const Expr *IdxE = ASE->getIdx();
     unsigned int Idx = 0;
-    llvm::APSInt Result;
+    Expr::EvalResult Result;
     if (IdxE && IdxE->EvaluateAsInt(Result, *Context)) {
       // this will truncate a possible uint64 value to uint32 value
-      Idx = (unsigned int)(*Result.getRawData());
+      llvm::APSInt IVal = Result.Val.getInt();
+      Idx = (unsigned int)(*IVal.getRawData());
     }
     BaseE = ASE->getBase()->IgnoreParenCasts();
     ASE = dyn_cast<ArraySubscriptExpr>(BaseE);
@@ -269,7 +270,8 @@ const Expr *Transformation::getMemberExprBaseExprAndIdxs(
   while (ME) {
     ValueDecl *VD = ME->getMemberDecl();
     FieldDecl *FD = dyn_cast<FieldDecl>(VD);
-    TransAssert(FD && "Bad FD!\n");
+    if (!FD)
+      return NULL;
     unsigned int Idx = FD->getFieldIndex();
     Idxs.push_back(Idx);
 
@@ -288,11 +290,12 @@ const Expr *Transformation::getMemberExprBaseExprAndIdxs(
 bool Transformation::isCXXMemberExpr(const MemberExpr *ME)
 {
   const ValueDecl *VD = ME->getMemberDecl();
-  if (dyn_cast<CXXMethodDecl>(VD))
-    return true;
 
   const FieldDecl *FD = dyn_cast<FieldDecl>(VD);
-  TransAssert(FD && "Bad FieldDecl!");
+  // VD can be either CXXMethodDecl, EnumConstantDecl or
+  // VarDecl (static data member)
+  if (!FD)
+    return true;
   const CXXRecordDecl *CXXRD = dyn_cast<CXXRecordDecl>(FD->getParent());
   if (!CXXRD)
     return false;
@@ -307,6 +310,8 @@ const Expr *Transformation::getMemberExprElem(const MemberExpr *ME)
 
   IndexVector Idxs;
   const Expr *BaseE = getMemberExprBaseExprAndIdxs(ME, Idxs);
+  if (!BaseE)
+    return NULL;
   return getInitExprFromBase(BaseE, Idxs);
 }
 
@@ -385,11 +390,12 @@ const Expr *Transformation::getBaseExprAndIdxs(const Expr *E,
       const ArraySubscriptExpr *ASE = dyn_cast<ArraySubscriptExpr>(E);
       const Expr *IdxE = ASE->getIdx();
       unsigned int Idx = 0;
-      llvm::APSInt Result;
+      Expr::EvalResult Result;
 
       // If we cannot have an integeral index, use 0.
       if (IdxE && IdxE->EvaluateAsInt(Result, *Context)) {
-        std::string IntStr = Result.toString(10);
+        llvm::APSInt IVal = Result.Val.getInt();
+        std::string IntStr = IVal.toString(10);
         std::stringstream TmpSS(IntStr);
         if (!(TmpSS >> Idx))
           TransAssert(0 && "Non-integer value!");
@@ -430,12 +436,13 @@ const Type *Transformation::getBasePointerElemType(const Type *Ty)
 
 int Transformation::getIndexAsInteger(const Expr *E)
 {
-  llvm::APSInt Result;
-  int Idx;
+  Expr::EvalResult Result; 
   if (!E->EvaluateAsInt(Result, *Context))
     TransAssert(0 && "Failed to Evaluate index!");
 
-  Idx = (int)(*Result.getRawData());
+  int Idx;
+  llvm::APSInt IVal = Result.Val.getInt();
+  Idx = (int)(*IVal.getRawData());
   return Idx;
 }
 
@@ -506,6 +513,9 @@ const FunctionDecl *Transformation::lookupFunctionDeclFromBases(
         const CXXRecordDecl *CXXRD,
         DeclContextSet &VisitedCtxs)
 {
+  if (!CXXRD->hasDefinition()) {
+    return NULL;
+  }
   for (CXXRecordDecl::base_class_const_iterator I =
        CXXRD->bases_begin(), E = CXXRD->bases_end(); I != E; ++I) {
 
@@ -734,6 +744,12 @@ const CXXRecordDecl *Transformation::getBaseDeclFromType(const Type *Ty)
     return getBaseDeclFromType(PT);
   }
 
+  case Type::Pointer: {
+    const PointerType *PT = dyn_cast<PointerType>(Ty);
+    const Type *PTy = PT->getPointeeType().getTypePtr();
+    return getBaseDeclFromType(PTy);
+  }
+
   case Type::SubstTemplateTypeParm: {
     const SubstTemplateTypeParmType *TP =
       dyn_cast<SubstTemplateTypeParmType>(Ty);
@@ -769,6 +785,7 @@ const CXXRecordDecl *Transformation::getBaseDeclFromType(const Type *Ty)
   case Type::SubstTemplateTypeParmPack:
   case Type::PackExpansion:
   case Type::Vector:
+  case Type::ExtVector:
   case Type::Builtin: // fall-through
     return NULL;
 
@@ -1059,7 +1076,18 @@ bool Transformation::isInIncludedFile(const Decl *D) const
 
 bool Transformation::isInIncludedFile(const Stmt *S) const
 {
-  return isInIncludedFile(S->getLocStart());
+  return isInIncludedFile(S->getBeginLoc());
+}
+
+bool Transformation::isDeclaringRecordDecl(const RecordDecl *RD)
+{
+  SourceLocation SemiLoc =
+    Lexer::findLocationAfterToken(RD->getSourceRange().getEnd(),
+                                  tok::semi,
+                                  *SrcManager,
+                                  Context->getLangOpts(),
+                                  /*SkipTrailingWhitespaceAndNewLine=*/true);
+  return SemiLoc.isInvalid();
 }
 
 Transformation::~Transformation(void)
