@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013, 2014, 2015 2016 The University of Utah
+// Copyright (c) 2012, 2013, 2014, 2015, 2016, 2018 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -57,8 +57,17 @@ typedef enum {
   TransToCounterTooBigError
 } TransformationError;
 
+namespace clang_delta_common_visitor {
+
+template<typename T> class CommonRenameClassRewriteVisitor;
+
+}
+
 class Transformation : public clang::ASTConsumer {
 
+template<typename T>
+friend class clang_delta_common_visitor::CommonRenameClassRewriteVisitor;
+
 public:
 
   Transformation(const char *TransName, const char *Desc)
@@ -290,6 +299,8 @@ protected:
 
   bool isInIncludedFile(const clang::Stmt *S) const;
 
+  bool isDeclaringRecordDecl(const clang::RecordDecl *RD);
+
   const std::string Name;
 
   int TransformationCounter;
