@@ -1,6 +1,6 @@
 ## -*- mode: Perl -*-
 ##
-## Copyright (c) 2012, 2013, 2015 The University of Utah
+## Copyright (c) 2012, 2013, 2015, 2016 The University of Utah
 ## All rights reserved.
 ##
 ## This file is distributed under the University of Illinois Open Source
@@ -32,11 +32,6 @@ sub advance ($$$) {
     return \$index;
 }
 
-sub advance_on_success ($$$) {
-    (my $cfile, my $arg, my $state) = @_;
-    return $state;
-}
-
 sub junk ($) {
     (my $list) = @_;
     my @stuff = split /,/, $list;
