@@ -1,6 +1,6 @@
-#!/bin/bash
+#!/usr/bin/env bash
 ##
-## Copyright (c) 2012 The University of Utah
+## Copyright (c) 2012, 2015, 2016 The University of Utah
 ## All rights reserved.
 ##
 ## This file is distributed under the University of Illinois Open Source
@@ -8,10 +8,20 @@
 
 ###############################################################################
 
+file="file2.c"
+
+CLANG="${CLANG:-clang}"
+GCC="${GCC:-gcc}"
+
+if [ $# -ne 0 ]; then
+  echo "usage: $0" 1>&2
+  exit 1
+fi
+
 rm -f out*.txt
 
 if 
-  clang -pedantic -Wall -O0 small.c  >out.txt 2>&1 &&\
+  "$CLANG" -pedantic -Wall -O0 "$file" >out.txt 2>&1 &&\
   ! grep 'incompatible redeclaration' out.txt &&\
   ! grep 'ordered comparison between pointer' out.txt &&\
   ! grep 'eliding middle term' out.txt &&\
@@ -26,7 +36,7 @@ if
   ! grep 'incompatible pointer to' out.txt &&\
   ! grep 'incompatible integer to' out.txt &&\
   ! grep 'type specifier missing' out.txt &&\
-  gcc -c -Wall -Wextra -O small.c  >outa.txt 2>&1 &&\
+  "$GCC" -c -Wall -Wextra -O "$file" >outa.txt 2>&1 &&\
   ! grep uninitialized outa.txt &&\
   ! grep 'control reaches end' outa.txt &&\
   ! grep 'no semicolon at end' outa.txt &&\
@@ -41,13 +51,13 @@ if
   ! grep 'incompatible implicit' outa.txt &&\
   ! grep 'excess elements in struct initializer' outa.txt &&\
   ! grep 'comparison between pointer and integer' outa.txt &&\
-  grep '\+\+' small.c
+  grep '\+\+' "$file"
 then
   exit 0
 else
   exit 1
 fi
 
-#  clang --analyze small.c > out_analyze.txt 2>&1 &&\
+#  "$CLANG" --analyze "$file" > out_analyze.txt 2>&1 &&\
 #  ! grep garbage out_analyze.txt &&\
 #  ! grep undefined out_analyze.txt &&\
